module LifeGameExample

//	**************************************************************************************************
//
//	This is the LifeGame program.
//
//	The program has been written in Clean 1.3.1 and uses the Clean Standard Object I/O library 1.1
//	
//	**************************************************************************************************

import StdEnv, StdIO
import Life, Help

::	Life
	=	{	gen	:: Generation
		,	size:: CellSize
		}
initialLife
	=	{	gen	= makeGeneration
		,	size= StartCellSize
		}
::	NoState
	=	NoState

Start :: *World -> *World
Start world
	#	(ids,world)	= openIds 7 world
	=	startIO initialLife NoState [initIO ids] [ProcessHelp (showAbout "Life" "LifeHelp")] world

initIO :: [Id] (PSt Life .p) -> PSt Life .p
initIO [commandsMenuID,eraseID,playID,haltID,stepID,windowID,timerID] life
	=	snd	(seqList	[	openWindow undef window
						,	openTimer  undef timer
						,	openMenu   undef file
						,	openMenu   undef options
						,	openMenu   undef commands
						]	life
			)
where
//	window defines the window that displays the universe and its inhabitants.
	window	= Window "Life" NilLS
				[	WindowId			windowID
				,	WindowClose			(noLS closeProcess)
				,	WindowMouse			onlyMouseDown Able (noLS1 track)
				,	WindowViewDomain	(getViewDomain StartCellSize)
				,	WindowOrigin		zero
				,	WindowHScroll 		(hscroll StartCellSize)
				,	WindowVScroll		(vscroll StartCellSize)
				,	WindowLook			(look initialLife)
				,	WindowResize
				]
	where
		hscroll dh viewframe {sliderThumb} move
			= case move of
				SliderIncSmall -> sliderThumb+dh
				SliderDecSmall -> sliderThumb-dh
				SliderIncLarge -> sliderThumb+(rectangleSize viewframe).w*9/10
				SliderDecLarge -> sliderThumb-(rectangleSize viewframe).w*9/10
				SliderThumb x  -> x
		vscroll dv viewframe {sliderThumb} move
			= case move of
				SliderIncSmall -> sliderThumb+dv
				SliderDecSmall -> sliderThumb-dv
				SliderIncLarge -> sliderThumb+(rectangleSize viewframe).h*9/10
				SliderDecLarge -> sliderThumb-(rectangleSize viewframe).h*9/10
				SliderThumb y  -> y
	
//	timer defines the timer that calculates subsequent life generations.
	timer	= Timer 0 NilLS
				[	TimerId				timerID
				,	TimerSelectState	Unable
				,	TimerFunction		(noLS1 (\_->step))
				]

//	file defines the "File" menu, containing only the quit command to terminate the program.
	file	= Menu "File"
				(	MenuItem "&Quit" [MenuShortKey 'q',MenuFunction (noLS closeProcess)]
				)	[]

//	options defines the "Options" menu to set the size of the displayed cells.
	options	= Menu "Options"
				(	SubMenu "Cell Size" 
		  			(	RadioMenu
		  				[	(title (2^i),Nothing,Just (char i),noLS (newsize (2^i)))
		  				\\	i<-[0..4]
						]	4 []
		  			)	[]
				)	[]
	where
		title size	= toString size +++ " * " +++ toString size
		char  i		= toChar (fromChar '1'+i)
	
//	commands defines the "Commands" menu to run and halt the computations of life generations.
	commands= Menu "Commands"
				(	MenuItem "&Erase Cells"	[MenuId eraseID,MenuShortKey 'e',MenuFunction (noLS erase)]
		  		:+:	MenuItem "&Play"		[MenuId playID, MenuShortKey 'p',MenuFunction (noLS play)]
				:+:	MenuItem "&Halt"		[MenuId haltID, MenuShortKey 'h',MenuFunction (noLS halt), MenuSelectState Unable]
				:+:	MenuItem "&Step"		[MenuId stepID, MenuShortKey 's',MenuFunction (noLS step)]
				)
				[	MenuId commandsMenuID	]
	
//	play starts the computation of successive generations given the current set of life cells.
	play :: (PSt Life .p) -> PSt Life .p
	play life
		= appListPIO
			[	disableWindowMouse	windowID
			,	setMenu				commandsMenuID
			[	disableMenuElements [eraseID,playID,stepID]
			,	enableMenuElements	[haltID]
			],	enableTimer			timerID
			]	life
	
//	halt stops the computation of successive generations, but does not change the current generation. 
	halt :: (PSt Life .p) -> PSt Life .p
	halt life
		= appListPIO
			[	enableWindowMouse	windowID
			,	setMenu				commandsMenuID
			[	disableMenuElements	[haltID]
			,	enableMenuElements	[eraseID,playID,stepID]
			],	disableTimer		timerID
			]	life
	
//	step calculates the next generation and displays it.
	step :: (PSt Life .p) -> PSt Life .p
	step life=:{ls=state=:{gen,size},io}
		#	state	= {state & gen=next}
		#	io		= drawInWindow  windowID render io
		#	io		= setWindowLook windowID False (look state) io
		=	{life & ls=state, io=io}
	where
		(next,died)	= lifeGame gen
		render		= drawCells (eraseCell size) died ++ drawCells (drawCell size) next
	
//	erase sets the current generation to empty and clears the window.
	erase :: (PSt Life .p) -> PSt Life .p
	erase life=:{ls=state,io}
		#	state	= {state & gen=makeGeneration}
		#	io		= setWindowLook windowID True (look state) io
		=	{life & ls=state, io=io}
	
//	newsize changes the size in which life cells are rendered and redraws the window.
	newsize :: Int (PSt Life .p) -> PSt Life .p
	newsize newSize life=:{ls=state=:{size=oldSize},io}
		#	state			= {state & size=newSize}
		#	(viewframe,io)	= getWindowViewFrame windowID io
  			oldOrigin		= viewframe.corner1
			newOrigin		= {x=oldOrigin.x/oldSize*newSize,y=oldOrigin.y/oldSize*newSize}
		#	io				= setWindowLook windowID False (look {state & gen=makeGeneration}) io
		#	io				= setWindowViewDomain windowID (getViewDomain newSize) io
		#	io				= moveWindowViewFrame windowID (toVector newOrigin-toVector oldOrigin) io
		#	io				= setWindowLook windowID True (look state) io
		=	{life & ls=state, io=io}
	
//	The window look:
	look :: Life SelectState UpdateState -> *Picture -> *Picture
	look {gen,size} _ {newFrame}
		=	seq [setPenColour Black,fill newFrame : drawCells (drawCell size) gen]
	
//	The window mouse accepts only MouseDown user actions:
	onlyMouseDown :: MouseState -> Bool
	onlyMouseDown (MouseDown _ _ _) = True
	onlyMouseDown (MouseDrag _ _)	= True
	onlyMouseDown _					= False
	
//	The window mouse action places and removes alive cells:
	track :: MouseState (PSt Life .p) -> PSt Life .p
	track mouse life=:{ls=state=:{gen,size},io}
		|	modifiers.commandDown
			#	state	= {state & gen=removeCell cell gen}
			#	io		= drawInWindow  windowID [eraseCell size cell] io
			#	io		= setWindowLook windowID False (look state) io
			=	 {life & ls=state, io=io}
		|	otherwise
			#	state	= {state & gen=insertCell cell gen}
			#	io		= drawInWindow  windowID [drawCell size cell] io
			#	io		= setWindowLook windowID False (look state) io
			=	{life & ls=state, io=io}
	where
		(pos,modifiers)	= case mouse of
							(MouseDown pos mods _) -> (pos,mods)
							(MouseDrag pos mods)   -> (pos,mods)
		cell			= makeLifeCell pos size
	
//	Given the size in which to render life cells, getViewDomain calculates the corresponding ViewDomain:
	getViewDomain :: CellSize -> ViewDomain
	getViewDomain size
		=	{corner1={x=size*left,y=size*top},corner2={x=size*right,y=size*bottom}}
	where
		{corner1={x=left,y=top},corner2={x=right,y=bottom}}	= Universe


//	Program constants.

Universe		:==	{corner1={x=(-1000),y=(-1000)},corner2={x=1000,y=1000}}
StartCellSize	:== 8
